#include <stdio.h>
#include <string.h>
#include <openssl/rand.h>

#include "buffer.h"

void
buffer_init (size_t length, Buffer *buffer)
{
    buffer->length = length;
    buffer->data = malloc (length);
}

void
buffer_init_random (size_t length, Buffer *buffer)
{
    buffer_init (length, buffer);
    RAND_bytes (buffer->data, length);
}

void
buffer_init_string (const char *value, Buffer *buffer)
{
    buffer_init (strlen (value), buffer);
    memcpy (buffer->data, value, buffer->length);
}

void
buffer_print (const char *name, Buffer buffer)
{
    size_t i;

    if (name)
        fprintf (stderr, "%s: ", name);
    for (i = 0; i < buffer.length; i++)
        fprintf (stderr, "%02X", buffer.data[i]);
    fprintf (stderr, "\n");
}

TPM_BOOL
buffer_matches (Buffer a, Buffer b)
{
    size_t i;

    if (a.length != b.length)
        return FALSE;

    for (i = 0; i < a.length; i++)
        if (a.data[i] != b.data[i])
            return FALSE;

    return TRUE;
}

void
buffer_free (Buffer buffer)
{
    size_t i;

    if (!buffer.data)
        return;

    for (i = 0; i < buffer.length; i++)
        buffer.data[i] = 0x00;
    free (buffer.data);

    buffer.data = NULL;
    buffer.length = 0;
}
