#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/wait.h>

#include "plymouth.h"

TPM_BOOL
plymouth_running (void)
{
    pid_t plymouth_pid = fork ();
    if (plymouth_pid == 0)
    {
        execlp ("plymouth", "plymouth", "--ping", NULL);
        _exit (EXIT_FAILURE);
    }
    int plymouth_status;
    if (waitpid (plymouth_pid, &plymouth_status, 0) < 0)
        return FALSE;

    return WIFEXITED (plymouth_status) && WEXITSTATUS (plymouth_status) == EXIT_SUCCESS;
}

TPM_BOOL
plymouth_prompt (const char *text, Buffer *buffer)
{
    int stdout_pipe[2];
    if (pipe (stdout_pipe) < 0)
    {
        fprintf (stderr, "Failed to make pipe: %s\n", strerror (errno));
        return FALSE;      
    }
    pid_t plymouth_pid = fork ();
    if (plymouth_pid < 0)
    {
        fprintf (stderr, "Failed to fork: %s\n", strerror (errno));
        close (stdout_pipe[0]);
        close (stdout_pipe[1]);
        return FALSE;
    }
    if (plymouth_pid == 0)
    {
        dup2 (stdout_pipe[1], STDOUT_FILENO);
        close (stdout_pipe[0]);
        close (stdout_pipe[1]);
        execlp ("plymouth", "plymouth", "ask-for-password", "--prompt", text, NULL);
        _exit (EXIT_FAILURE);
    }
    close (stdout_pipe[1]);
    // FIXME: Check full read
    buffer_init (1024, buffer);
    ssize_t n_read = read (stdout_pipe[0], buffer->data, buffer->length);
    close (stdout_pipe[0]);
    if (n_read < 0)
    {
        fprintf (stderr, "Failed to read from plymouth: %s\n", strerror (errno));
        return FALSE;
    }
    buffer->length = n_read;
    int plymouth_status;
    if (waitpid (plymouth_pid, &plymouth_status, 0) < 0)
    {
        fprintf (stderr, "Failed to run plymouth: %s\n", strerror (errno));
        return FALSE;
    }
    if (WIFEXITED (plymouth_status))
    {
        if (WEXITSTATUS (plymouth_status) != EXIT_SUCCESS)
        {
            fprintf (stderr, "plymouth exited with status %d\n", WEXITSTATUS (plymouth_status));
            return FALSE;
        }

        return TRUE;
    }
    else if (WIFSIGNALED (plymouth_status))
    {
        fprintf (stderr, "plymouth exited with signal %d\n", WTERMSIG (plymouth_status));
        return FALSE;
    }
    else
    {
        fprintf (stderr, "Failed to run plymouth: %s\n", strerror (errno));
        return FALSE;
    }
}
